package com.bananahammock.auth;

import android.content.Context;
import android.util.Log;
import com.bananahammock.Utils;
import org.apache.http.cookie.Cookie;

import java.io.*;
import java.util.*;

/**
 *  A simple cookie manager. This gives me full control over cookie persistence in the way I want.
 *
 * User: scott
 * Date: 8/29/11
 * Time: 9:04 PM
 */
public final class JSoupCookieManager {

	////////////////////////////////////////////////////////////
	//	Statics

    private static final String COOKIES_DIR_NAME = "cookies";

	////////////////////////////////////////////////////////////
	//	Variables

    private static JSoupCookieManager INSTANCE;

    private final Map<String, Cookie> cookies = new HashMap<String, Cookie>();

	////////////////////////////////////////////////////////////
	// Statics

    public static final JSoupCookieManager instance() {
        if (INSTANCE == null) {
            Log.e(JSoupCookieManager.class.getSimpleName(), "Please call init() before trying to use this class.");
            throw new RuntimeException(JSoupCookieManager.class.getSimpleName() + ": Please call init() before trying to use this class.");
        }
        return INSTANCE;
    }

    public static final void init(Context appContext) {
        if (INSTANCE == null) {
            INSTANCE = new JSoupCookieManager();
        }

        File cookiesDir = ensureCacheDir(appContext);

        File[] files = cookiesDir.listFiles();
        List<Cookie> cookies = new ArrayList<Cookie>(files.length);
        for (File f : files) {
            ObjectInputStream ois = null;
            try {
                // yeah, yeah, I know. Serializing an object to a file is a horrible idea.
                // i'm fine with it right now because I don't care if they fail to de-serialize.
                // if they do fail, we are just going to fail the log in because we don't have the
                // proper cookies and the user will get asked to log in again. no big deal.
                ois = new ObjectInputStream(new FileInputStream(f));
                SerializableCookie serialCookie = (SerializableCookie) ois.readObject();
                cookies.add(serialCookie.fromSerialForm());
            } catch (Exception ex) {
                Log.d(JSoupCookieManager.class.getSimpleName(), "Invalid cookie found in cookie jar", ex);
            } finally {
                Utils.close(ois);
            }
        }

        // if any cookie is expired we are just going to require a new round of them
        for (Cookie cookie : cookies) {
            if (cookie.isExpired(new Date())) {
                return;
            }
        }

        // the cookies aren't expired, so we are actually going to use them
        for (Cookie cookie : cookies) {
            INSTANCE.addCookie(cookie);
        }
    }

    public static final void flushToDisk(Context appContext) {
        // do nothing if we have no cookies
        if (INSTANCE.cookies.isEmpty()) {
            return;
        }

        File cookiesDir = ensureCacheDir(appContext);
        // out with the old, in with the new
        if (cookiesDir.listFiles().length > 0) {
            if (Utils.deleteChildren(cookiesDir) == false) {
                Log.e(JSoupCookieManager.class.getSimpleName(), "Unable to clear cache on the way out!");
                throw new RuntimeException(JSoupCookieManager.class.getSimpleName() + ": Unable to clear cache on the way out!");
            }
        }
        Map<String, Cookie> cookies = INSTANCE.cookies;
        for (Map.Entry<String, Cookie> e : cookies.entrySet()) {
            String name = e.getKey();
            Cookie cookie = e.getValue();
            ObjectOutputStream oos = null;
            try {
                File f = new File(cookiesDir, name + "_" + cookie.hashCode());
                oos = new ObjectOutputStream(new FileOutputStream(f));
                SerializableCookie serialCookie = new SerializableCookie(cookie);
                oos.writeObject(serialCookie);
            } catch (Exception ex) {
                Log.e(JSoupCookieManager.class.getSimpleName(), "Unable to write the cookie to disk. The file couldn't be created.", ex);
            } finally {
                Utils.close(oos);
            }
        }
    }

    ////////////////////////////////////////////////////////////
	//	Private Static Methods

    private static final File ensureCacheDir(Context appContext) {
        File cookiesDir = new File(appContext.getFilesDir(), COOKIES_DIR_NAME);
        if (cookiesDir.exists() == false) {
            cookiesDir.mkdirs();
        }
        return cookiesDir;
    }

	////////////////////////////////////////////////////////////
	//	Constructor

    private JSoupCookieManager() {}

	////////////////////////////////////////////////////////////
	//	Public Methods

    public final void addCookie(Cookie cookie) {
        this.cookies.put(cookie.getName(), cookie);
    }

    public final Set<String> getKeys() {
        return Collections.unmodifiableSet(this.cookies.keySet());
    }

    public final String getValue(String key) {
        Cookie cookie = this.cookies.get(key);
        return cookie == null ? "" : cookie.getValue();
    }

    public final boolean hasValidCookies() {
        if (this.cookies.isEmpty()) {
            return false;
        }

        Date rightNow = new Date();
        for (Map.Entry<String, Cookie> e : cookies.entrySet()) {
            Cookie cookie = e.getValue();
            boolean expired = cookie.isExpired(rightNow);
            if (expired) {
                return false;
            }
        }
        return true;
    }

}
